from chronometry.progress import ProgressBar
from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists

COMPANY = 'company'


def get_company_list_guides(wikipedia=None, echo=0):
	"""
	:param wikipedia:
	:param echo: int or ProgressBar or bool
	:rtype: dict[str, WikipediaListGuide]
	"""
	return {
		'record_label': WikipediaListGuide(
			list_type='list_of_lists', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'record_label'],
			urls='https://en.wikipedia.org/wiki/List_of_record_labels'
		),
		'lists_of_airlines': WikipediaListGuide(
			list_type='list_of_lists', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'airline'],
			columns={COMPANY: 'airline'},
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_airlines'
			]
		),
		'public_company': WikipediaListGuide(
			list_type='list_of_lists', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'public_company'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_companies_by_stock_exchange_listing',
				'https://en.wikipedia.org/wiki/FTSE_100_Index',
				'https://en.wikipedia.org/wiki/FTSE_250_Index'
			],
			columns={COMPANY: ['issuer_name', COMPANY, 'english_name', 'stock_name', 'short_name', 'full_name']}
		),
		'american_newspapers_lists': WikipediaListGuide(
			list_type='list_of_lists', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia,
			categories=[COMPANY, 'newspaper', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_newspapers_in_the_United_States',
			sections=['By state and territory'],
			columns={COMPANY: ['newspaper', 'name', 'title']},
			exclude_urls=[
				'https://en.wikipedia.org/wiki/List_of_newspapers_in_Michigan',
				'https://en.wikipedia.org/wiki/List_of_newspapers_in_Rhode_Island'
			]
		),
		'automotive_fuel_retail': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'automotive_fuel_retail'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_automotive_fuel_retailers'
		),
		'sports_car_manufacturers': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'car_manufacturer', 'sports_car_manufacturer'],
			representation='table',
			columns={COMPANY: 'manufacturer'},
			urls='https://en.wikipedia.org/wiki/List_of_automotive_fuel_retailers'
		),
		'bicycle': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bicycle'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_bicycle_brands_and_manufacturing_companies'
		),
		'bicycle_part': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bicycle_part'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_bicycle_part_manufacturing_companies'
		),
		'car_audio': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'car_audio'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_car_audio_manufacturers_and_brands'
		),
		'defunct': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'defunct'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_defunct_consumer_brands'
		),
		'digital_camera': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'digital_camera'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_digital_camera_brands'
		),
		'electronics': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'electronics'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_electronics_brands',
			sections=['Asia', 'Europe', 'North America', 'Oceania', 'South America']),
		'italian': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'italian'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Italian_brands'
		),
		'mexican': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'mexican'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Mexican_brands'
		),
		'microbrewery': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'microbrewery'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_microbreweries'
		),
		'mountaineering_equipment': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'mountaineering_equipment'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_mountaineering_equipment_brands'
		),
		'piano': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'piano'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_piano_brand_names'
		),
		'motorcycle': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'motorcycle'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_motorcycle_manufacturers'
		),
		'sewing_machine': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'sewing_machine'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_sewing_machine_brands'
		),
		'video_telecommunication': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'video_telecommunication'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_video_telecommunication_services_and_product_brands'
		),
		'soft_drink': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'soft_drink'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_soft_drink_producers'
		),
		'frozen_food': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'frozen_food'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_frozen_food_brands'
		),
		'bakery': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bakery'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_bakeries'
		),
		'cleaning': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'cleaning'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_cleaning_companies'
		),
		'named_after_people': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'named_after_people'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_companies_named_after_people'
		),
		'conglomerates': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'conglomerates'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_conglomerates'
		),
		'sony_subsidiary': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'sony_subsidiary'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Sony_subsidiaries'
		),
		'canadian_national_railway': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'canadian_national_railway'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Canadian_National_Railways_companies'
		),
		'canadian_pacific_railway_subsidiary': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'canadian_pacific_railway_subsidiary'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_subsidiary_railways_of_the_Canadian_Pacific_Railway'
		),
		'drive_in_theater': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'drive_in_theater'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_drive-in_theaters'
		),
		'elevator': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'elevator'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_elevator_manufacturers'
		),
		'employee_owned': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'employee_owned'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_employee-owned_companies'
		),
		'private_equity': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'private_equity'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_private_equity_firms'
		),
		'etymology': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'etymology'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_company_name_etymologies'
		),
		'franchises': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'franchises'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_franchises'
		),
		'holding': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'holding'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_holding_companies'
		),
		'reestablished': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'reestablished'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_re-established_companies'
		),
		'switched_industry': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'switched_industry'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_companies_that_switched_industries'
		),
		'government_owned': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'government_owned'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_government-owned_companies'
		),
		'multinational': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'multinational'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_multinational_corporations'
		),
		'past_ftse_100': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'past_ftse_100'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/FTSE_100_Index', sections='Past constituents'
		),
		'advertising': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'advertising'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_advertising_technology_companies'
		),
		'animation_distribution': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'animation_distribution'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_animation_distribution_companies'
		),
		'anime': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'anime'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_anime_companies'
		),
		'asset_management': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'asset_management'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_asset_management_firms'
		),
		'latin_american_broadcasting': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'latin_american_broadcasting'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_broadcasting_companies_in_Latin_America'
		),
		'cable_tv': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'cable_tv'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_cable_television_companies'
		),
		'casino': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'casino'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_casinos'
		),
		'clock': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'clock'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_clock_manufacturers'
		),
		'computer_hardware': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'computer_hardware'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_computer_hardware_manufacturers'
		),
		'computer_system': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'computer_system'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_computer_system_manufacturers'
		),
		'concentrating_solar_thermal_power': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'concentrating_solar_thermal_power'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_concentrating_solar_thermal_power_companies'
		),
		'book_distributor': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'book_distributor'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_book_distributors'
		),
		'film_distributor': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'film_distributor'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_film_distributors_by_country'
		),
		'duty_free': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'duty_free'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_duty-free_shops'
		),
		'frozen_custard': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'frozen_custard'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_frozen_custard_companies'
		),
		'enterprise_search_vendor': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'enterprise_search_vendor'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_enterprise_search_vendors',
			sections='Vendors of prop'
		),
		'north_american_filling_station': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'north_american', 'filling_station'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_filling_station_chains_in_North_America'
		),
		'food': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'food'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_food_companies'
		),
		'defunct_gambling': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'defunct_gambling'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_defunct_gambling_companies'
		),
		'home_video': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'home_video'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_home_video_companies'
		),
		'food_truck': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'food_truck'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_food_trucks'
		),
		'ice': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'ice'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_ice_companies'
		),
		'seafood': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'seafood'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_seafood_companies'
		),
		'teahouse': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'teahouse'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_teahouses'
		),
		'trading': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'trading'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_trading_companies'
		),
		'management_consulting': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'management_consulting'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_management_consulting_firms'
		),
		'marketing_research': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'marketing_research'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_marketing_research_firms'
		),
		'modeling_agency': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'modeling_agency'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_modeling_agencies'
		),
		'multilevel_marketing': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'multilevel_marketing'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_multi-level_marketing_companies'
		),
		'american_mutual_fund': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'american', 'mutual_fund'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_mutual-fund_families_in_the_United_States'
		),
		'open_game_license_publisher': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'open_game_license_publisher'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_OGL_publishers'
		),
		'petroleum': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'petroleum'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_oil_exploration_and_production_companies'
		),
		'oilfield_service': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'oilfield_service'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_oilfield_service_companies'
		),
		'pharmaceutical': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'pharmaceutical'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_pharmaceutical_companies'
		),
		'pornographic_studio': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'pornographic_studio'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_pornographic_film_studios'
		),
		'defunct_fast_food': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'defunct_fast_food'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_defunct_fast-food_restaurant_chains'
		),
		'private_security': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'private_security'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_private_security_companies'
		),
		'freight_ship': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'freight_ship'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_freight_ship_companies'
		),
		'passenger_ship': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'passenger_ship'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_passenger_ship_companies'
		),
		'silicon': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'silicon'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_silicon_producers'
		),
		'steel': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'steel'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_steel_producers'
		),
		'system_on_a_chip': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'system_on_a_chip'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_system-on-a-chip_suppliers'
		),
		'tea': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'tea'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_tea_companies'
		),
		'television_network': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'television_network'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_television_networks_by_country'
		),
		'winery': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'winery'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_vineyards_and_wineries'
		),
		'aircraft': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'aircraft'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers:_A',
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers:_B-C',
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers:_D-G',
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers_H-L',
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers:_M-P',
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers:_Q-S',
				'https://en.wikipedia.org/wiki/List_of_aircraft_manufacturers:_T-Z',
			]
		),
		'rotorcraft': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'rotorcraft'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_rotorcraft_manufacturers_by_country'
		),
		'helicopter_airline': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'helicopter_airline'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_helicopter_airlines'
		),
		'bank': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bank'],
			columns={COMPANY: ['bank_name']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_banks_(alphabetical)',
				'https://en.wikipedia.org/wiki/List_of_banks_in_India'
			]
		),
		'investment_bank': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'investment_bank'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_investment_banks'
		),
		'bookstore': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bookstore'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_independent_bookstores_in_the_United_States',
				'https://en.wikipedia.org/wiki/List_of_independent_bookstores'
			]
		),
		'bookstore_chain': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bookstore_chain'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_bookstore_chains'
		),
		'ice_cream_parlor': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'ice_cream_parlor'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_ice_cream_parlor_chains'
		),
		'resaurant_chain': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'resaurant_chain'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_restaurant_chains',
				'https://en.wikipedia.org/wiki/List_of_fast_food_restaurant_chains',
				'https://en.wikipedia.org/wiki/List_of_pizza_chains',
				'https://en.wikipedia.org/wiki/List_of_pizza_franchises'
			]
		),
		'law_firm': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['law_firm', 'european'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_largest_Europe-based_law_firms_by_revenue'
		),
		'tv_channel': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'tv_channel'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_movie_television_channels',
				'https://en.wikipedia.org/wiki/List_of_generalist_television_channels',
				'https://en.wikipedia.org/wiki/List_of_music_video_television_channels'
			]
		),
		'hotel': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'hotel'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_chained-brand_hotels'
		),
		'laptop': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'laptop'],
			representation='table', columns={COMPANY: ['name', 'brand']},
			urls='https://en.wikipedia.org/wiki/List_of_laptop_brands_and_manufacturers',
			sections=['Brands', 'Other brands']),
		'ski': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'ski'],
			representation='table', columns={COMPANY: 'name_of_brand'},
			urls='https://en.wikipedia.org/wiki/List_of_ski_brands'
		),
		'swimwear': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'swimwear'],
			representation='table', columns={COMPANY: 'brand'},
			urls='https://en.wikipedia.org/wiki/List_of_swimwear_brands'
		),

		'copper': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'copper'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_copper_production_by_company'
		),
		'fortune_global_500': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'fortune_global_500'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/Fortune_Global_500'
		),
		'financial_services': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'financial_services'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_financial_services_companies_by_revenue'
		),
		'largest_internet': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'internet'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_Internet_companies',
			sections='List'
		),
		'largest_technology': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'technology'],
			representation='table', columns={COMPANY: 'company_2'},
			urls='https://en.wikipedia.org/wiki/List_of_largest_technology_companies_by_revenue',
			sections='List'
		),
		'largest_software': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'software'],
			representation='table', columns={COMPANY: 'organization_2'},
			urls='https://en.wikipedia.org/wiki/List_of_the_largest_software_companies',
			sections='Forbes Global 2000'
		),
		'private_equity_table': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'private_equity'],
			representation='table', columns={COMPANY: ['private_equity_firm', 'firm']},
			urls='https://en.wikipedia.org/wiki/List_of_private_equity_firms'
		),
		'involved_in_holocaust': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'involved_in_holocaust'],
			representation='table', columns={COMPANY: 'company_name'},
			urls='https://en.wikipedia.org/wiki/List_of_companies_involved_in_the_Holocaust'
		),
		'oldest': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'old'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_oldest_companies'
		),
		's_and_p_400': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 's_and_p_400'],
			representation='table', columns={COMPANY: 'security'},
			urls='https://en.wikipedia.org/wiki/List_of_S%26P_400_companies'
		),
		's_and_p_500': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 's_and_p_500'],
			representation='table', columns={COMPANY: 'security'},
			urls='https://en.wikipedia.org/wiki/List_of_S%26P_500_companies'
		),
		'largest_by_revenue': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'high_revenue'],
			representation='table', columns={COMPANY: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_largest_companies_by_revenue'
		),
		'largest_european_manufacturing': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['manufacturing', 'european'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_European_manufacturing_companies_by_revenue'
		),
		'ftse_100': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'ftse_100'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/FTSE_100_Index'
		),
		'unicorn_startup': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['startup', 'unicorn_startup'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_unicorn_startup_companies'
		),
		'animation_studio': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'animation_studio'],
			representation='table', columns={COMPANY: 'studio'},
			urls='https://en.wikipedia.org/wiki/List_of_animation_studios'
		),
		'largest_biotechnology': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'biotechnology'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_biotechnology_and_pharmaceutical_companies'
		),
		'bullion_dealer': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'bullion_dealer'],
			representation='table', columns={COMPANY: 'dealer'},
			urls='https://en.wikipedia.org/wiki/List_of_bullion_dealers'
		),
		'computer_aided_technology': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'computer_aided_technology'],
			representation='table', columns={COMPANY: 'company_name'},
			urls='https://en.wikipedia.org/wiki/List_of_CAx_companies'
		),
		'champagne_house': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'champagne_house'],
			representation='table', columns={COMPANY: 'house'},
			urls='https://en.wikipedia.org/wiki/List_of_Champagne_houses'
		),
		'coffee': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'coffee'],
			representation='table', columns={COMPANY: 'company_name'},
			urls='https://en.wikipedia.org/wiki/List_of_coffee_companies'
		),
		'electronic_design_automation': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'electronic_design_automation'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_EDA_companies'
		),
		'film_production': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'film_production'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_live-action_film_production_companies'
		),
		'former_investment_bank': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['former', 'investment_bank'],
			representation='table', columns={COMPANY: 'firm'},
			urls='https://en.wikipedia.org/wiki/List_of_investment_banks',
			sections=['Notable former investment banks and brokerages']
		),
		'largest_chemical_producer': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'chemical_producer'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_chemical_producers'
		),
		'mobile_network': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'mobile_network'],
			representation='table', columns={COMPANY: [COMPANY, 'operator']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_mobile_network_operators',
				'https://en.wikipedia.org/wiki/List_of_mobile_network_operators_of_Europe',
				'https://en.wikipedia.org/wiki/List_of_mobile_network_operators_of_the_Americas',
				'https://en.wikipedia.org/wiki/List_of_mobile_network_operators_of_the_Asia_Pacific_region',
				'https://en.wikipedia.org/wiki/List_of_mobile_network_operators_of_the_Middle_East_and_Africa',
				'https://en.wikipedia.org/wiki/List_of_Canadian_mobile_phone_companies',
				'https://en.wikipedia.org/wiki/List_of_mobile_network_operators_of_The_Caribbean',
				'https://en.wikipedia.org/wiki/List_of_United_States_wireless_communications_service_providers'
			]
		),

		'canadian_mutual_fund': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['canadian', 'mutual_fund'],
			representation='table', columns={COMPANY: 'ultimate_parent'},
			urls='https://en.wikipedia.org/wiki/List_of_mutual-fund_families_in_Canada'
		),
		'largest_oil_and_gas': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'oil_and_gas'],
			representation='table', columns={COMPANY: 'company_name'},
			urls='https://en.wikipedia.org/wiki/List_of_largest_oil_and_gas_companies_by_revenue'
		),
		'photo_voltaic': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'photo_voltaic'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_photovoltaics_companies'
		),
		'pornography': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'pornography'],
			representation='table', columns={COMPANY: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_pornography_companies'
		),
		'european_power': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['european', 'power'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_European_power_companies_by_carbon_intensity',
		),
		'container_shipping': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'container_shipping'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_container_shipping_companies'
		),
		'private_spaceflight': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['spaceflight', 'private'],
			representation='table', columns={COMPANY: 'company_name'},
			urls='https://en.wikipedia.org/wiki/List_of_private_spaceflight_companies'
		),
		'steel_table': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'steel'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_steel_producers'
		),
		'telephone': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'telephone'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_telephone_operating_companies'
		),
		'venture_capital': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'venture_capital'],
			representation='table', columns={COMPANY: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_venture_capital_firms'
		),
		'video_game_developer': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['video_game', 'video_game_developer'],
			representation='table', columns={COMPANY: 'developer'},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_video_game_developers',
				'https://en.wikipedia.org/wiki/List_of_PlayStation_3_games_released_on_disc'
			]
		),
		'video_game_publisher': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['video_game', 'video_game_publisher'],
			representation='table', columns={COMPANY: 'publisher'},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_video_game_publishers',
				'https://en.wikipedia.org/wiki/List_of_video_games_considered_the_best'
			]
		),
		'indie_game_developer': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=['video_game', 'indie_game_developer'],
			representation='table', columns={COMPANY: 'developer'},
			urls='https://en.wikipedia.org/wiki/List_of_indie_game_developers'
		),
		'flag_carrier': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'flag_carrier'],
			representation='table', columns={COMPANY: 'airline'},
			urls='https://en.wikipedia.org/wiki/Flag_carrier'
		),
		'airline': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'airline'],
			columns={COMPANY: 'airline'},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_airlines_with_more_than_100_destinations',
				'https://en.wikipedia.org/wiki/World%27s_largest_airlines',
				'https://en.wikipedia.org/wiki/List_of_low-cost_airlines',
				'https://en.wikipedia.org/wiki/List_of_regional_airlines',
				'https://en.wikipedia.org/wiki/List_of_airlines_of_Australia'
			]
		),
		'coffeehouse': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'coffeehouse'],
			representation='table', columns={COMPANY: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_coffeehouse_chains'
		),
		'restaurant_chain': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'restaurant_chain'],
			representation='table', columns={COMPANY: 'name'},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_restaurant_chains',
				'https://en.wikipedia.org/wiki/List_of_casual_dining_restaurant_chains'
			]
		),
		'law_firm_table': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'law_firm'],
			representation='table', columns={COMPANY: ['firm', 'name', 'firm_name']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_largest_law_firms_by_revenue',
				'https://en.wikipedia.org/wiki/List_of_largest_Canada-based_law_firms_by_revenue',
				'https://en.wikipedia.org/wiki/List_of_largest_United_States-based_law_firms_by_head_count',
				'https://en.wikipedia.org/wiki/List_of_largest_Japan-based_law_firms_by_head_count',
				'https://en.wikipedia.org/wiki/List_of_US_law_firms_by_profits_per_partner'
			]
		),

		'tv_channel_table': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'tv_channel'],
			representation='table', columns={COMPANY: ['name', 'network']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_documentary_television_channels',
				'https://en.wikipedia.org/wiki/List_of_news_television_channels',
			]
		),
		'forbes_global_2000': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'forbes_global_2000'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/Forbes_Global_2000'
		),
		'largest_nordic': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'nordic'],
			representation='table', columns={COMPANY: COMPANY},
			urls='https://en.wikipedia.org/wiki/List_of_largest_Nordic_companies'
		),
		'mobile_phone_maker': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'mobile_phone_maker'],
			representation='table', columns={COMPANY: 'brand'},
			urls='https://en.wikipedia.org/wiki/List_of_mobile_phone_brands_by_country'),
		'stock_exchange': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[COMPANY, 'stock_exchange'],
			columns={COMPANY: ['stock_exchange', 'exchange']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_stock_exchanges',
				'https://en.wikipedia.org/wiki/List_of_stock_exchanges_in_the_Americas',
				'https://en.wikipedia.org/wiki/List_of_Asian_stock_exchanges',
				'https://en.wikipedia.org/wiki/List_of_European_stock_exchanges',
				'https://en.wikipedia.org/wiki/List_of_African_stock_exchanges',
				'https://en.wikipedia.org/wiki/List_of_stock_exchanges_in_Oceania'
			]
		),
		'newspapers': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia,
			categories=[COMPANY, 'newspaper', 'british_newspaper', 'british'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_newspapers_in_the_United_Kingdom'
			],
			columns={COMPANY: ['title']}
		),
		'american_newspapers': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia,
			categories=[COMPANY, 'newspaper', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_newspapers_in_the_United_States',
			sections=['Top 10 newspapers by circulation', 'Longest running newspaper'],
			columns={COMPANY: ['newspaper']}
		),
		'american_companies': WikipediaListGuide(
			list_type='list', data_type=COMPANY, echo=echo - 1,
			wikipedia=wikipedia,
			categories=[COMPANY, 'american'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_companies_of_the_United_States_by_state',
				'https://en.wikipedia.org/wiki/List_of_Alabama_companies',
				'https://en.wikipedia.org/wiki/List_of_Alaska_companies',
				'https://en.wikipedia.org/wiki/List_of_Arizona_companies',
				'https://en.wikipedia.org/wiki/List_of_Arkansas_companies',
				'https://en.wikipedia.org/wiki/List_of_California_companies',
				'https://en.wikipedia.org/wiki/List_of_Colorado_companies',
				'https://en.wikipedia.org/wiki/List_of_Connecticut_companies',
				'https://en.wikipedia.org/wiki/List_of_Delaware_companies',
				'https://en.wikipedia.org/wiki/List_of_Florida_companies',
				'https://en.wikipedia.org/wiki/List_of_Georgia_(U.S._state)_companies',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_Idaho',
				'https://en.wikipedia.org/wiki/List_of_Illinois_companies',
				'https://en.wikipedia.org/wiki/List_of_Kansas_companies',
				'https://en.wikipedia.org/wiki/List_of_Kentucky_companies',
				'https://en.wikipedia.org/wiki/List_of_Massachusetts_companies',
				'https://en.wikipedia.org/wiki/List_of_Michigan_companies',
				'https://en.wikipedia.org/wiki/List_of_Minnesota_companies',
				'https://en.wikipedia.org/wiki/List_of_North_Dakota_companies',
				'https://en.wikipedia.org/wiki/List_of_companies_in_Greater_Cincinnati',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_Oklahoma_City',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_Tulsa,_Oklahoma',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_Oregon',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_the_Harrisburg_area',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_the_Philadelphia_area',
				'https://en.wikipedia.org/wiki/List_of_corporations_in_Pittsburgh',
				'https://en.wikipedia.org/wiki/List_of_Rhode_Island_companies',
				'https://en.wikipedia.org/wiki/List_of_companies_based_in_Nashville,_Tennessee',
				'https://en.wikipedia.org/wiki/List_of_Texas_companies',
				'https://en.wikipedia.org/wiki/List_of_Utah_companies',
				'https://en.wikipedia.org/wiki/List_of_companies_headquartered_in_Northern_Virginia',
				'https://en.wikipedia.org/wiki/List_of_Washington_(state)_companies',
				'https://en.wikipedia.org/wiki/List_of_Wyoming_companies',
				'https://en.wikipedia.org/wiki/List_of_restaurants_in_Hawaii',
				'https://en.wikipedia.org/wiki/List_of_airlines_of_Hawaii'
			],
			columns={COMPANY: ['name', 'corporation']}
		)
	}

def get_companies(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	list_guides = get_company_list_guides(wikipedia=wikipedia, echo=progress_bar)
	return WikipediaLists(wikipedia_lists=list_guides, categories=[COMPANY], echo=progress_bar)



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

ORGANIZATION = 'organization'


def get_ngo_guides(wikipedia=None, echo=0):

	urls_and_categories = [
		('https://en.wikipedia.org/wiki/Category:Non-governmental_organizations', []),
		('https://en.wikipedia.org/wiki/Category:Anti-corruption_non-governmental_organizations', ['anti_corruption']),
		('https://en.wikipedia.org/wiki/Category:International_nongovernmental_organizations', ['international']),
		(
			'https://en.wikipedia.org/wiki/Category:Non-governmental_organizations_involved_in_the_Israeli%E2%80%93Palestinian_conflict',
			['involved_in_israel_palestine_conflict']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Non-governmental_organizations_involved_in_the_Israeli%E2%80%93Palestinian_peace_process',
			['involved_in_israel_palestine_conflict']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Non-governmental_organizations_involved_in_the_Israeli%E2%80%93Syrian_conflict',
			['involved_in_israel_syria_conflict']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Non-governmental_organizations_involved_in_the_Turkish_War_of_Independence',
			['involved_in_turkish_war_of_independence']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Non-governmental_organizations_with_consultative_status_at_the_United_Nations',
			['united_nations_consultative_ngo']
		),
		('https://en.wikipedia.org/wiki/Category:World_War_II_non-governmental_organizations', ['world_war_2_ngo']),
		('https://en.wikipedia.org/wiki/Category:Non-governmental_organization_stubs', ['ngo_stub']),
		('https://en.wikipedia.org/wiki/International_non-governmental_organization', ['international']),
		('https://en.wikipedia.org/wiki/List_of_non-governmental_organisations_in_India', ['india']),
		('https://en.wikipedia.org/wiki/List_of_non-governmental_organizations_in_China', ['china']),
		('https://en.wikipedia.org/wiki/List_of_human_rights_organisations', ['human_rights_ngo']),
		('https://en.wikipedia.org/wiki/List_of_environmental_organizations', ['environmental_ngo']),
		('https://en.wikipedia.org/wiki/List_of_non-governmental_organizations_in_Pakistan', ['pakistan']),
		('https://en.wikipedia.org/wiki/List_of_non-governmental_organisations_in_Bangladesh', ['bangladesh']),
		(
			'https://en.wikipedia.org/wiki/List_of_non-governmental_organisations_based_in_Karachi',
			['pakistan', 'karachi']
		),
		('https://en.wikipedia.org/wiki/List_of_youth_organizations', ['youth_organization']),
		('https://en.wikipedia.org/wiki/List_of_non-governmental_organisations_in_Kenya', ['kenya']),
		(
			'https://en.wikipedia.org/wiki/List_of_organizations_that_combat_human_trafficking',
			['anti_human_trafficking_ngo']
		),
		('https://en.wikipedia.org/wiki/List_of_non-governmental_organizations_in_Vietnam', ['vietnam']),
		('https://en.wikipedia.org/wiki/List_of_women%27s_organizations', ['womens_organization']),
		('https://en.wikipedia.org/wiki/List_of_international_organizations_based_in_Geneva', ['switzerland', 'geneva'])
	]

	return {
		(urls[0] if isinstance(urls, list) else urls): WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'ngo'] + categories,
			urls=urls, sections=['Pages in category'], columns={ORGANIZATION: ['name', 'name_of_ngo']}
		)
		for urls, categories in urls_and_categories
	}


def get_charity_guides(wikipedia=None, echo=0):

	charities_urls_and_categories = [
		('https://en.wikipedia.org/wiki/List_of_charitable_foundations', []),
		('https://en.wikipedia.org/wiki/Category:Charities_based_in_Canada', ['canada']),
		(
			'https://en.wikipedia.org/wiki/Category:Charities_based_in_Washington,_D.C.',
			['united_states', 'washington_dc']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Animal_charities_based_in_the_United_States',
			['united_states', 'animal_charity']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Children%27s_charities_based_in_the_United_States',
			['united_states', 'children_charity']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Development_charities_based_in_the_United_States',
			['united_states', 'development_charity']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Educational_charities_based_in_the_United_States',
			['united_states', 'educational_charity']
		),
		('https://en.wikipedia.org/wiki/Category:Foundations_based_in_the_United_States', ['united_states']),
		(
			'https://en.wikipedia.org/wiki/Category:Health_charities_in_the_United_States',
			['united_states', 'health_charity']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Prison_charities_based_in_the_United_States',
			['united_states', 'prison_charity']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Religious_charities_based_in_the_United_States',
			['united_states', 'religious_charity']
		),
		(
			'https://en.wikipedia.org/wiki/Category:Social_welfare_charities_based_in_the_United_States',
			['united_states', 'social_welfare_charity']
		)
	]

	charities = {
		(urls[0] if isinstance(urls, list) else urls): WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1,
			data_type=ORGANIZATION, categories=[ORGANIZATION, 'charity'] + categories,
			urls=urls,
			sections=['Pages in category'],
			representation='list'
		)
		for urls, categories in charities_urls_and_categories
	}

	by_continent_urls_and_categories = [
		('https://en.wikipedia.org/wiki/Category:Charities_based_in_Europe', ['europe']),
		('https://en.wikipedia.org/wiki/Category:Charities_based_in_Africa', ['africa']),
		('https://en.wikipedia.org/wiki/Category:Charities_based_in_Asia', ['asia']),
		('https://en.wikipedia.org/wiki/Category:Charities_based_in_North_America', ['north_america']),
		('https://en.wikipedia.org/wiki/Category:Charities_based_in_South_America', ['south_america'])
	]
	by_continent = {
		(urls[0] if isinstance(urls, list) else urls): WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'charity'] + categories,
			sections=['Subcategories'], representation='list', urls=urls
		)
		for urls, categories in by_continent_urls_and_categories
	}

	return {**by_continent, **charities}


def get_univesity_guides(wikipedia=None, echo=0):

	international = WikipediaListGuide(
		list_type='list_of_lists',
		wikipedia=wikipedia, echo=echo - 1, categories=[ORGANIZATION, 'academic', 'university'], data_type=ORGANIZATION,
		urls=[
			'https://en.wikipedia.org/wiki/Lists_of_universities_and_colleges_by_country',
			'https://en.wikipedia.org/wiki/Lists_of_American_institutions_of_higher_education'
		],
		exclude_urls=[
			'https://en.wikipedia.org/wiki/Lists_of_American_institutions_of_higher_education',
			'https://en.wikipedia.org/wiki/Anton_de_Kom_University_of_Suriname',
			'https://en.wikipedia.org/wiki/List_of_universities_in_India',
			'https://en.wikipedia.org/wiki/University_of_Greenland',
			'https://en.wikipedia.org/wiki/Universitat_d%27Andorra',
			'https://en.wikipedia.org/wiki/University_of_Gibraltar',
			'https://en.wikipedia.org/wiki/List_of_universities_in_Greece',
			'https://en.wikipedia.org/wiki/University_of_Luxembourg',
			'https://en.wikipedia.org/wiki/International_University_of_Monaco',
			'https://en.wikipedia.org/wiki/List_of_schools_in_Papua_New_Guinea'
		],
		columns={ORGANIZATION: [
			'university', 'institution', 'university_name', 'name_of_university', 'school', 'name_in_english',
			'unofficial_name_in_english', 'university_full_name', 'university_college', 'institute_name', 'name',
			'universities', 'english_name'
		]}

	)
	india = WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[ORGANIZATION, 'academic', 'university', 'indian'], data_type=ORGANIZATION,
		urls=[
			'https://en.wikipedia.org/wiki/Institutes_of_National_Importance',
			'https://en.wikipedia.org/wiki/Central_university_(India)',
			'https://en.wikipedia.org/wiki/List_of_deemed_universities',
			'https://en.wikipedia.org/wiki/List_of_private_universities_in_India',
			'https://en.wikipedia.org/wiki/List_of_state_universities_in_India'
		],
		representation='table',
		columns={ORGANIZATION: ['university', 'institution']}
	)
	return {'international_universities': international, 'indian_universities': india}


def get_temple_guides(wikipedia=None, echo=0):

	churches = WikipediaListGuide(
		list_type='list_of_lists',
		wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'religious', 'church'],
		urls=[
			'https://en.wikipedia.org/wiki/Category:Lists_of_churches',
			'https://en.wikipedia.org/wiki/Lists_of_Armenian_churches',
			'https://en.wikipedia.org/wiki/Lists_of_cathedrals'
		],
		sections=['Pages in category'],
		exclude_urls=[
			'https://en.wikipedia.org/wiki/List_of_churches_in_Albania',
			'https://en.wikipedia.org/wiki/List_of_Buddhist_temples',
			'https://en.wikipedia.org/wiki/Lists_of_cathedrals',
			'https://en.wikipedia.org/wiki/List_of_churches_of_the_Jesuit_Missions_of_the_Chiquitos',
			'https://en.wikipedia.org/wiki/Friends_meeting_house',
			'https://en.wikipedia.org/wiki/Greek_Catholic_Church',
			'https://en.wikipedia.org/wiki/List_of_churches_in_North_Macedonia'
		],
		columns={ORGANIZATION: [
			'church_temple', 'name', 'church', 'cathedral_church', 'building', 'temple_name_english',
			'name_of_church'
		]}
	)
	buddhist = WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'religious', 'buddhist_temple'],
		urls=[
			'https://en.wikipedia.org/wiki/List_of_Buddhist_temples'
		],
		representation='list'
	)
	return {'churches': churches, 'buddhist_temples': buddhist}


def get_political_party_guides(wikipedia=None, echo=0):

	return {
		'political_parties_in_india': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'political_party', 'india'],
			urls='https://en.wikipedia.org/wiki/List_of_political_parties_in_India',
			columns={ORGANIZATION: 'name'}
		),
		'political_parties_in_the_united_kingdom': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'political_party', 'united_kingdom'],
			urls='https://en.wikipedia.org/wiki/List_of_political_parties_in_the_United_Kingdom',
			columns={ORGANIZATION: 'party'}
		),
		'political_parties_in_the_united_states': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'political_party', 'united_states'],
			urls='https://en.wikipedia.org/wiki/List_of_political_parties_in_the_United_States',
			columns={ORGANIZATION: 'party'}
		),
		'largest_political_parties': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'political_party'],
			urls='https://en.wikipedia.org/wiki/List_of_largest_political_parties',
			columns={ORGANIZATION: 'name'}
		),
		'green_political_parties': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'political_party', 'green_party'],
			urls='https://en.wikipedia.org/wiki/List_of_green_political_parties',
			representation='list'
		),
		'generic_names_of_political_parties': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'political_party'],
			urls='https://en.wikipedia.org/wiki/List_of_generic_names_of_political_parties',
			exclude_urls=[
				'https://en.wikipedia.org/wiki/Green_Party',
				'https://en.wikipedia.org/wiki/Industrial_Workers_of_the_World',
				'https://en.wikipedia.org/wiki/Radical_Party_of_the_Left'
			],
			columns={ORGANIZATION: ['party']}
		),
		'outlaw_motorcycle_clubs': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, 'outlaw', 'motorcycle_club'],
			urls='https://en.wikipedia.org/wiki/List_of_outlaw_motorcycle_clubs',
			columns={ORGANIZATION: ['name']}
		)
	}


def get_terrorist_group_guides(wikipedia=None, echo=0):

	return {'terrorist_groups': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'terrorist_group'],
		urls='https://en.wikipedia.org/wiki/List_of_designated_terrorist_groups',
		columns={ORGANIZATION: 'organization'}
	)}


def get_sports_team_guides(wikipedia=None, echo=0):
	CLUB = 'sporting_club'
	return {
		'sports_teams_of_the_united_states_and_canada': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, CLUB, 'north_america'],
			urls='https://en.wikipedia.org/wiki/Major_professional_sports_teams_of_the_United_States_and_Canada',
			columns={ORGANIZATION: 'team'}, sections=['Teams']
		),
		'national_basketball_association': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, CLUB, 'basketball_team', 'nba_club'],
			urls='https://en.wikipedia.org/wiki/National_Basketball_Association',
			columns={ORGANIZATION: 'team'}, sections=['Teams']
		),
		'relocated_national_basketball_association_teams': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION,
			categories=[ORGANIZATION, CLUB, 'basketball_club', 'nba_club', 'relocated_club'],
			urls='https://en.wikipedia.org/wiki/List_of_relocated_National_Basketball_Association_teams',
			sections=['Relocated teams']
		),
		'association_football_club': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'association_football_club'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_CAF_countries',
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_AFC_countries',
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_UEFA_countries',
				'https://en.wikipedia.org/wiki/List_of_second_division_football_clubs_in_UEFA_countries',
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_CONCACAF_countries',
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_OFC_countries',
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_CONMEBOL_countries',
				'https://en.wikipedia.org/wiki/List_of_top-division_football_clubs_in_non-FIFA_countries'
			],
			columns={ORGANIZATION: ['club', 'team']}
		),
		'cricket_teams': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'cricket_club'],
			urls='https://en.wikipedia.org/wiki/List_of_current_first-class_cricket_teams',
			columns={ORGANIZATION: 'name'}
		)
	}


def get_intergovernmental(wikipedia=None, echo=0):

	return {'intergovernmental_organizations': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, data_type=ORGANIZATION, categories=[ORGANIZATION, 'intergovernmental'],
		urls=[
			'https://en.wikipedia.org/wiki/List_of_intergovernmental_organizations'
		],
		exclude_pages=[
			'https://en.wikipedia.org/wiki/Eurasia',
			'https://en.wikipedia.org/wiki/Mediterranean_Sea',
			'https://en.wikipedia.org/wiki/Indian_Ocean',
			'https://en.wikipedia.org/wiki/Arctic_Ocean',
			'https://en.wikipedia.org/wiki/Pacific_Ocean',
			'https://en.wikipedia.org/wiki/African,_Caribbean_and_Pacific_Group_of_States'
		],
		representation='list'
	)}


def get_organizations(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			**get_charity_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_ngo_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_temple_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_political_party_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_terrorist_group_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_univesity_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_sports_team_guides(wikipedia=wikipedia, echo=progress_bar),
			**get_intergovernmental(wikipedia=wikipedia, echo=progress_bar)

		}
	)



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

CHARACTER = 'fictional_character'


def get_fictional_characters(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			'starwars_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'starwars_character'],
				urls='https://en.wikipedia.org/wiki/List_of_Star_Wars_characters',
				columns={CHARACTER: 'name'}
			),
			'pokemon_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'pokemon_character'],
				urls='https://en.wikipedia.org/wiki/List_of_generation_I_Pok%C3%A9mon',
				columns={CHARACTER: 'english_name'}
			),
			'coronation_street_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'coronation_street_character'],
				urls='https://en.wikipedia.org/wiki/List_of_Coronation_Street_characters',
				columns={CHARACTER: 'character'}
			),
			'emmerdale_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'emmerdale_character'],
				urls='https://en.wikipedia.org/wiki/List_of_Emmerdale_characters',
				columns={CHARACTER: 'character'}
			),
			'neighbours_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'neighbours_character'],
				urls='https://en.wikipedia.org/wiki/List_of_Neighbours_characters',
				columns={CHARACTER: 'character'}
			),
			'the_simpsons_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'the_simpsons_character'],
				urls='https://en.wikipedia.org/wiki/List_of_The_Simpsons_characters',
				columns={CHARACTER: 'character'}
			),
			'hollyoaks_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'hollyoaks_character'],
				urls='https://en.wikipedia.org/wiki/List_of_Hollyoaks_characters',
				columns={CHARACTER: 'character'}
			),
			'x_men_characters': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CHARACTER,
				categories=[CHARACTER, 'x_men_character'],
				urls='https://en.wikipedia.org/wiki/List_of_X-Men_members',
				columns={CHARACTER: 'character'}
			)
		}
	)



from .SpecialPage import SpecialPage

from silverware import Link
import pandas as pd


class IosDevices(SpecialPage):
	def __init__(self, wikipedia=None, page=None):
		super().__init__(wikipedia=wikipedia, url='https://en.wikipedia.org/wiki/List_of_iOS_devices', page=page)
		self._load_extra_stuff()

	def _load_extra_stuff(self):
		def get_main_tables(x):
			result = []
			for table in x:
				if 'announced' in table.columns and 'released' in table.columns:
					result.append(table)
			return pd.concat(result).reset_index(drop=True)

		self.pensieve.store(
			key='ios_device_table', precursors='tables', function=get_main_tables,
			evaluate=False
		)

		def get_page_urls(x):
			result = []
			for link in list(x['model']):
				if isinstance(link, Link):
					if link.url.startswith('https://en.wikipedia.org/wiki/'):
						if link.url not in result:
							result.append(link.url)
			return result

		self.pensieve.store(
			key='ios_device_page_urls', precursors='ios_device_table', function=get_page_urls,
			evaluate=False
		)



from .IosDevices import IosDevices
from .ListPage import ListPage
from .WikipediaList import WikipediaList
from .WikipediaLists import WikipediaLists
from .WikipediaEntities import WikipediaEntities
from .signature import PageSignature


from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar


MYTH = 'mythological_figure'


def get_supernatural_beings(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			'germanic_deities': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=MYTH,
				categories=[MYTH, 'deity', 'germanic', 'germanic_deity'],
				urls='https://en.wikipedia.org/wiki/List_of_Germanic_deities',
				columns={MYTH: ['name']}
			),
			'greek_mythological_figures': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=MYTH,
				categories=[MYTH, 'greek', 'greek_mythological_figure'],
				urls='https://en.wikipedia.org/wiki/List_of_Greek_mythological_figures'
			),
			'hindu_creatures': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=MYTH,
				categories=[MYTH, 'hindu', 'hindu_mythological_figure'],
				urls='https://en.wikipedia.org/wiki/List_of_legendary_creatures_in_Hindu_mythology'
			),
			'roman_deities': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=MYTH,
				categories=[MYTH, 'roman', 'roman_deity'],
				urls=['https://en.wikipedia.org/wiki/List_of_Roman_deities']
			),
			'legendary_creatures': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=MYTH,
				categories=[MYTH, 'legendary_creature'],
				urls='https://en.wikipedia.org/wiki/Lists_of_legendary_creatures',
				representation='list'
			)
		}
	)



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists

from chronometry.progress import ProgressBar


PERSON = 'person'
POLITICIAN = 'politician'
PRIME_MINISTER = 'prime_minister'
MONARCH = 'monarch'
MUSICIAN = 'musician'
ARTIST = 'artist'
ATHLETE = 'athlete'

def get_person_list_guides(wikipedia=None, echo=0):
	result = {
		'religion_founders': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'religion_founder'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_founders_of_religious_traditions'
			],
			representation='table',
			columns={'person': ['name']}
		),
		'female_models': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'model'],
			urls=[
				'https://en.wikipedia.org/wiki/Category:American_female_models'
			],
			representation='list'
		),
		'tv_presenters': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'tv_presenter'],
			urls='https://en.wikipedia.org/wiki/List_of_television_presenters',
			representation='list'
		),
		'youtubers': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'youtuber'],
			urls='https://en.wikipedia.org/wiki/List_of_YouTubers',
			columns={PERSON: ['user']}
		),
		'writers': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'writer'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_writers'
			],
			columns={PERSON: ['name', 'author']},
			exclude_urls=['https://en.wikipedia.org/wiki/Category:Self-help_books']
		),
		'essayists': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'essayist'],
			urls='https://en.wikipedia.org/wiki/List_of_essayists',
			columns={PERSON: 'name'}
		),
		'american_writers': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'writer', 'american'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_American_writers'
			],
			columns={PERSON: ['name', 'author']},
			exclude_urls=['https://en.wikipedia.org/wiki/Category:Self-help_books']
		),
		'novelists_by_century': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'writer', 'australian'],
			urls=[
				'https://en.wikipedia.org/wiki/Category:Australian_novelists_by_century',
				'https://en.wikipedia.org/wiki/Category:Australian_writers_by_genre'
			],
			columns={PERSON: ['name', 'author']},
			exclude_urls=['https://en.wikipedia.org/wiki/Category:Self-help_books']
		),
		'english_writers': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'writer', 'english'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_English_writers'
			],
			columns={PERSON: ['name', 'author']},
			exclude_urls=['https://en.wikipedia.org/wiki/Category:Self-help_books']
		),
		'slovak_authors': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'writer', 'slovak'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_Slovak_authors'
			],
			columns={PERSON: ['name', 'author']},
			exclude_urls=['https://en.wikipedia.org/wiki/Category:Self-help_books']
		),
		'sailors': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'sailor'],
			urls='https://en.wikipedia.org/wiki/List_of_sailors',
			representation='list'
		),
		'sea_captains': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'sailor', 'sea_captain'],
			urls='https://en.wikipedia.org/wiki/List_of_sea_captains',
			columns={PERSON: ['about']}
		),
		'pirates': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'sailor', 'pirate'],
			urls='https://en.wikipedia.org/wiki/List_of_pirates',
			columns={PERSON: ['name']}
		),
		'spies_by_nationality': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'spy'],
			urls='https://en.wikipedia.org/wiki/Category:Spies_by_nationality',
			representation='list'
		),
		'american_spies': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'spy', 'american'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_American_spies',
			],
			columns={PERSON: 'person'}
		),
		'british_spies': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'spy', 'british'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_British_spies',
			],
			columns={PERSON: 'person'}
		),
		'spies_in_world_war_ii': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'spy', 'world_war_2'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_spies_in_World_War_II'
			],
			columns={PERSON: 'person'}
		),
		'astronauts_by_name': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'astronaut'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_astronauts_by_name'
			],
			representation='list'
		),
		'chief_executive_officers': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'executive', 'ceo'],
			urls='https://en.wikipedia.org/wiki/List_of_chief_executive_officers',
			columns={PERSON: 'executive'}
		),
		'chief_financial_officers': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'executive', 'cfo'],
			urls='https://en.wikipedia.org/wiki/Category:Chief_financial_officers',
			sections=['Pages in category'],
			representation='list'
		),
		'jewish_american_businesspeople': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'business_person', 'jewish', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_Jewish_American_businesspeople',
			representation='list'
		),
		'physicists': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'physicist'],
			urls='https://en.wikipedia.org/wiki/List_of_physicists',
			representation='list'
		),
		'mathematicians': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'mathematician'],
			urls='https://en.wikipedia.org/wiki/Lists_of_mathematicians',
			representation='list'
		),
		'chemists': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'chemist'],
			urls='https://en.wikipedia.org/wiki/List_of_chemists',
			representation='list'
		),
		'economists': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'economist'],
			urls='https://en.wikipedia.org/wiki/List_of_economists',
			representation='list'
		),
		'data_scientists': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'data_scientist'],
			urls='https://en.wikipedia.org/wiki/Category:Data_scientists',
			representation='list',
			sections='Pages in category'
		),
		'computer_scientists': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'computer_scientist'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_computer_scientists',
				'https://en.wikipedia.org/wiki/List_of_pioneers_in_computer_science'
			],
			columns={PERSON: 'person'}
		),
		'programmers': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'programmer'],
			urls='https://en.wikipedia.org/wiki/List_of_programmers',
			representation='list'
		),
		'astronomers': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'scientist', 'astronomer'],
			urls='https://en.wikipedia.org/wiki/List_of_astronomers',
			representation='list'
		),
		'philosophers': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'philosopher'],
			urls='https://en.wikipedia.org/wiki/Lists_of_philosophers',
			exclude_urls=[
				'https://en.wikipedia.org/wiki/Christian_philosophy',
				'https://en.wikipedia.org/wiki/Continental_philosophy',
				'https://en.wikipedia.org/wiki/Index_of_Eastern_philosophy_articles',
				'https://en.wikipedia.org/wiki/Anarchism',
				'https://en.wikipedia.org/wiki/Pragmatism',
				'https://en.wikipedia.org/wiki/List_of_Armenian_scientists',
				'https://en.wikipedia.org/wiki/List_of_people_from_the_Basque_Country',
				'https://en.wikipedia.org/wiki/List_of_Jewish_scientists'
			],
			columns={PERSON: ['name']}
		),
		'serial_killers': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'serial_killer'],
				urls=[
				'https://en.wikipedia.org/wiki/List_of_serial_killers_by_number_of_victims',
				'https://en.wikipedia.org/wiki/List_of_serial_killers_by_country',
				'https://en.wikipedia.org/wiki/List_of_serial_killers_in_the_United_States',
				'https://en.wikipedia.org/wiki/List_of_serial_killers_before_1900'
			],
			columns={PERSON: 'name'}
		),
		'richest_people': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'richest_in_the_world'],
				urls='https://en.wikipedia.org/wiki/Lists_of_people_by_net_worth',
			sections = ['By nationality', 'By region'],
			columns={PERSON: ['name']}
		),
		'prime_ministers_of_the_united_kingdom': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, PRIME_MINISTER, 'british'],
			urls='https://en.wikipedia.org/wiki/List_of_Prime_Ministers_of_the_United_Kingdom_by_tenure',
			representation='table',
			columns={PERSON: ['prime_minister']}
		),
		'prime_ministers_of_canada': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, PRIME_MINISTER, 'canadian'],
			urls='https://en.wikipedia.org/wiki/List_of_Prime_Ministers_of_Canada',
			representation='table',
			columns={PERSON: ['name']}
		),
		'presidents_of_the_philippines': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'president', 'filipino'],
			urls='https://en.wikipedia.org/wiki/List_of_Presidents_of_the_Philippines',
			columns={PERSON: ['name']}
		),
		'presidents_of_the_united_states': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'president', 'american'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_Presidents_of_the_United_States_by_previous_experience',
				'https://en.wikipedia.org/wiki/List_of_Presidents_of_the_United_States_by_home_state'
			],
			representation='table',
			columns={PERSON: ['president_2', 'president']}
		),
		'members_of_the_united_states_congress': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'member_of_congress', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_members_of_the_United_States_Congress_by_longevity_of_service',
			columns={PERSON: 'name'}
		),
		'vice_presidents_of_the_united_states': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'vice_president', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_Vice_Presidents_of_the_United_States',
			columns={PERSON: 'vice_president'}
		),
		'united_states_senators': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'senator', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_current_United_States_Senators',
			columns={PERSON: ['officer', 'senator']}
		),
		'justices_of_the_supreme_court_of_the_united_states': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'justice', 'supreme_court_justice', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_Justices_of_the_Supreme_Court_of_the_United_States',
			columns={PERSON: ['justice']}
		),
		'members_of_the_united_states_house_of_representatives': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'member_of_parliament', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_current_members_of_the_United_States_House_of_Representatives',
			columns={PERSON: ['officer', 'member']}
		),
		'united_states_governors': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'governor', 'american'],
			urls='https://en.wikipedia.org/wiki/List_of_United_States_governors',
			columns={PERSON: 'governor'}
		),
		'chinese_leaders': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'leader', 'chinese'],
			urls='https://en.wikipedia.org/wiki/List_of_Chinese_leaders',
			columns={PERSON: ['name']}
		),
		'indian_politicians': WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, POLITICIAN, 'indian'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_Prime_Ministers_of_India',
				'https://en.wikipedia.org/wiki/List_of_current_Indian_chief_ministers',
				'https://en.wikipedia.org/wiki/List_of_current_Indian_governors',
				'https://en.wikipedia.org/wiki/List_of_current_Indian_chief_justices',
				'https://en.wikipedia.org/wiki/List_of_current_Indian_legislative_speakers_and_chairmen',
				'https://en.wikipedia.org/wiki/List_of_current_Indian_opposition_leaders',
				'https://en.wikipedia.org/wiki/List_of_Chief_Justices_of_India',
				'https://en.wikipedia.org/wiki/List_of_sitting_judges_of_the_Supreme_Court_of_India',
				'https://en.wikipedia.org/wiki/List_of_former_judges_of_the_Supreme_Court_of_India',
				'https://en.wikipedia.org/wiki/List_of_chief_ministers_from_the_Communist_Party_of_India_(Marxist)',
				'https://en.wikipedia.org/wiki/List_of_chief_ministers_from_the_Indian_National_Congress',
				'https://en.wikipedia.org/wiki/List_of_chief_ministers_from_the_Bharatiya_Janata_Party',
				'https://en.wikipedia.org/wiki/List_of_current_Indian_deputy_chief_ministers'
			],
			columns={PERSON: ['name', 'chief_justice', 'speaker', 'opposition_leader', 'name_of_the_justice']}
		),
		'nazis': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'nazi'],
			urls='https://en.wikipedia.org/wiki/List_of_Nazis',
			representation='list'
		),
		'gay_lesbian_or_bisexual_people': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, 'lgbt_person'],
			urls='https://en.wikipedia.org/wiki/List_of_gay,_lesbian_or_bisexual_people',
			columns={PERSON: ['name']}
		),
		'persons_by_nationality': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=PERSON,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_people_by_nationality',
				'https://en.wikipedia.org/wiki/Lists_of_people_from_African_Union_states',
				'https://en.wikipedia.org/wiki/Lists_of_people_from_India_by_state',
				'https://en.wikipedia.org/wiki/Lists_of_New_Zealanders',
				'https://en.wikipedia.org/wiki/Category:New_Zealand_M%C4%81ori_people',
				'https://en.wikipedia.org/wiki/List_of_Sri_Lankan_people',
				'https://en.wikipedia.org/wiki/List_of_Swedish_people',
				'https://en.wikipedia.org/wiki/Lists_of_Indigenous_Australians',
				'https://en.wikipedia.org/wiki/Lists_of_Inuit',
				'https://en.wikipedia.org/wiki/List_of_Tamil_people'
			],
			exclude_urls=[
				'https://en.wikipedia.org/wiki/Filipinos',
				'https://en.wikipedia.org/wiki/Malagasy_people',
				'https://en.wikipedia.org/wiki/List_of_New_Zealand_child_actors',
				'https://en.wikipedia.org/wiki/Blogging_in_New_Zealand',
				'https://en.wikipedia.org/wiki/Congregation_of_Christian_Brothers_in_New_Zealand',
				'https://en.wikipedia.org/wiki/Surinamese_people',
				'https://en.wikipedia.org/wiki/Tibetan_people',
				'https://en.wikipedia.org/wiki/List_of_Huaorani_people',
				'https://en.wikipedia.org/wiki/Sindhis',
				'https://en.wikipedia.org/wiki/Category:Lists_of_fictional_Chinese_people',
				'https://en.wikipedia.org/wiki/Disciples_of_Confucius',
				'https://en.wikipedia.org/wiki/Four_Beauties',
				'https://en.wikipedia.org/wiki/Four_Lords_of_the_Warring_States',
				'https://en.wikipedia.org/wiki/Falkland_Islanders',
				'https://en.wikipedia.org/wiki/Boer',
				'https://en.wikipedia.org/wiki/Han_Chinese'

			],
			columns={PERSON: [
				'name', 'posthumous_name', 'tokugawas_2', 'prime_minister', 'member_1', 'player',
				'name_1', 'title', 'name_by_which_most_commonly_known', 'laureate_3', 'president',
				'first_lady_gentleman'
			]}
		),
		'actors': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, ARTIST, 'actor'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_actors'
			],
			sections=['Nationality'],
			columns={PERSON: ['name', 'actor_role', 'actor', ]}
		),
		'korean_actors': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, ARTIST, 'actor', 'south_korean'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_South_Korean_actors'
			],
			sections=['nationality'],
			columns={PERSON: ['name', 'actor_role', 'actor', ]}
		),
		'musicians': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, ARTIST, MUSICIAN],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_musicians'
			],
			sections=['Instrument', 'Location or nationality'],
			columns={PERSON: ['musician', 'name', 'artist_name', 'band', 'artist_s', 'artist', 'band_name']}
		),
		'fiddlers': WikipediaListGuide(
			list_type='list_of_lists', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, ARTIST, MUSICIAN, 'fiddler'],
			urls='https://en.wikipedia.org/wiki/List_of_fiddlers',
			columns={PERSON: 'name_of_fiddler'}
		)
	}

	artist_urls_and_categories = [
		('https://en.wikipedia.org/wiki/List_of_architects', ['architect']),
		('https://en.wikipedia.org/wiki/List_of_animators', ['animator']),
		('https://en.wikipedia.org/wiki/List_of_dancers', ['dancer']),
		('https://en.wikipedia.org/wiki/List_of_fashion_designers', ['fashion_designer']),
		('https://en.wikipedia.org/wiki/List_of_illustrators', ['illustrator']),
		('https://en.wikipedia.org/wiki/List_of_tattoo_artists', ['tattoo_artist']),
		('https://en.wikipedia.org/wiki/List_of_photographers', ['photographer']),
		('https://en.wikipedia.org/wiki/List_of_photojournalists', ['photojournalist', 'photographer']),
		('https://en.wikipedia.org/wiki/List_of_sculptors', ['sculptor']),
		('https://en.wikipedia.org/wiki/List_of_comedians', ['comedian']),
		('https://en.wikipedia.org/wiki/List_of_poets', ['poet']),
		('https://en.wikipedia.org/wiki/List_of_musicians_from_Chicago', ['musician', 'chicagoan']),
		('https://en.wikipedia.org/wiki/List_of_jazz_musicians', ['musician', 'jazz_musician'])
	]

	for urls, categories in artist_urls_and_categories:
		result[f'other_artists_{categories[0]}'] = WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, ARTIST] + categories,
			urls=urls,
			columns={PERSON: 'name'}
		)

	athlete_urls_and_categories = [
		('https://en.wikipedia.org/wiki/Lists_of_sportspeople', []),
		('https://en.wikipedia.org/wiki/Lists_of_Major_League_Baseball_players', ['baseball_player']),
		('https://en.wikipedia.org/wiki/Lists_of_National_Basketball_Association_players', ['basketball_player']),
		('https://en.wikipedia.org/wiki/Lists_of_American_football_players', ['american_football_player']),
		('https://en.wikipedia.org/wiki/List_of_Pro_Bowl_players', ['bowl_player']),
		('https://en.wikipedia.org/wiki/Lists_of_association_football_players', ['association_football_player']),
		('https://en.wikipedia.org/wiki/Lists_of_golfers', ['golfer']),
		('https://en.wikipedia.org/wiki/List_of_NHL_players', ['hockey_player']),
		('https://en.wikipedia.org/wiki/Lists_of_kickboxers', ['kickboxer']),
		('https://en.wikipedia.org/wiki/Category:Lists_of_rowers', ['rower']),
		('https://en.wikipedia.org/wiki/Category:Lists_of_skiers', ['skier']),
		('https://en.wikipedia.org/wiki/Category:Lists_of_female_skiers', ['skier', 'female']),
		('https://en.wikipedia.org/wiki/Category:Lists_of_male_skiers', ['skier', 'male']),
		('https://en.wikipedia.org/wiki/Category:Lists_of_alpine_skiers', ['skier']),
		('https://en.wikipedia.org/wiki/Category:Lists_of_ski_jumping_medalists', ['ski_jumper']),
		('https://en.wikipedia.org/wiki/Category:Racing_drivers_by_competition', ['racing_driver']),
		('https://en.wikipedia.org/wiki/Lists_of_tennis_players', ['tennis_player']),
		('https://en.wikipedia.org/wiki/Lists_of_wrestlers', ['wrestler']),
		('https://en.wikipedia.org/wiki/List_of_Formula_One_World_Drivers%27_Champions', ['racing_driver'])
	]

	for urls, categories in athlete_urls_and_categories:
		category = categories[0] if len(categories) > 0 else ''
		result[f'other_athletes_{category}'] = WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, ATHLETE] + categories,
			urls=urls,
			columns={PERSON: [
				'player', 'son_s', 'name', 'women', 'men', 'skater', 'gold', 'silver', 'bronze',
				'1st', '2nd', '3rd', 'winner', 'runner_up', 'third', 'singles_men', 'singles_women',
				'doubles_men', 'doubles_women', 'doubles_mixed', 'driver'
			]}
		)

	monarchs_dictionary = {
		'english': 'https://en.wikipedia.org/wiki/List_of_English_monarchs',
		'russian': 'https://en.wikipedia.org/wiki/List_of_Russian_monarchs',
		'polish': 'https://en.wikipedia.org/wiki/List_of_Polish_monarchs',
		'swedish': 'https://en.wikipedia.org/wiki/List_of_Swedish_monarchs',
		'ottoman': 'https://en.wikipedia.org/wiki/List_of_sultans_of_the_Ottoman_Empire',
		'saudi': 'https://en.wikipedia.org/wiki/King_of_Saudi_Arabia',
		'portuguese': 'https://en.wikipedia.org/wiki/List_of_Portuguese_monarchs'
	}

	for nationality, url in monarchs_dictionary.items():
		result[f'{nationality}_{MONARCH}s'] = WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, MONARCH, nationality, f'{nationality}_monarch'],
			columns={PERSON: ['name', 'sultan']}, urls=url
		)

	others_dictionary = {
		'freemason': [
			'https://en.wikipedia.org/wiki/List_of_Freemasons_(A%E2%80%93D)',
			'https://en.wikipedia.org/wiki/List_of_Freemasons_(E%E2%80%93Z)'
		],
		'atheist': [
			'https://en.wikipedia.org/wiki/List_of_atheists_in_science_and_technology',
			'https://en.wikipedia.org/wiki/List_of_atheist_authors',
			'https://en.wikipedia.org/wiki/List_of_atheist_philosophers'
		],
		'died_in_traffic_collision': 'https://en.wikipedia.org/wiki/List_of_people_who_died_in_traffic_collisions',
		'nobel_laureate': 'https://en.wikipedia.org/wiki/List_of_Nobel_laureates',
		'porn_star': 'https://en.wikipedia.org/wiki/List_of_pornographic_performers_by_decade',
		'leader_of_soviet_union': 'https://en.wikipedia.org/wiki/List_of_leaders_of_the_Soviet_Union'
	}

	for category, urls in others_dictionary.items():
		result[category] = WikipediaListGuide(
			list_type='list', data_type=PERSON, echo=echo - 1,
			wikipedia=wikipedia, categories=[PERSON, category], urls=urls,
			columns={PERSON: [
				'name', 'physics', 'chemistry', 'physiology_or_medicine', 'literature', 'peace', 'economics',
				'name_lifetime'
			]}
		)

	return result


def get_persons(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	list_guides = get_person_list_guides(wikipedia=wikipedia, echo=progress_bar)
	return WikipediaLists(wikipedia_lists=list_guides, categories=[PERSON], echo=progress_bar)



def get_paragraph_statistics(paragraph_tokens, paragraph_links):
	all_tokens = []
	all_links = []
	for tokens in paragraph_tokens:
		all_tokens += tokens

	for links in paragraph_links:
		all_links += links

	num_tokens = len(all_tokens)
	num_links = len(all_links)
	num_unique_tokens = len(set(all_tokens))
	num_unique_links = len(set(all_links))

	return {
		'num_tokens': num_tokens, 'num_links': num_links,
		'num_unique_tokens': num_unique_tokens, 'num_unique_links': num_unique_links,
		'links_to_tokens_ratio': num_links / num_tokens if num_tokens != 0 else -1,
		'unique_links_to_tokens_ratio': num_unique_links / num_tokens if num_tokens != 0 else -1,
		'unique_tokens_ratio': num_unique_tokens / num_tokens if num_tokens != 0 else 1
	}



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar


def get_diseases(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)

	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			'diseases': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, categories='disease',
				data_type='disease',
				urls='https://en.wikipedia.org/wiki/Lists_of_diseases',
				columns={'disease': ['common_name', 'disease', 'disorder']}
			)
		}
	)



from .WikipediaList import WikipediaList, merge_records, get_records_data
from .WikipediaListGuide import WikipediaListGuide
from pensieve import Pensieve
from pandas import DataFrame
from chronometry.progress import ProgressBar


class WikipediaLists:
	def __init__(self, wikipedia_lists, categories=None, echo=0):
		"""
		:type wikipedia_lists: dict[str, WikipediaList] or dict[str, WikipediaListGuide
		"""
		if not isinstance(wikipedia_lists, dict):
			raise TypeError('wikipedia_lists should be a dictionary')

		echo = max(0, echo)
		progress_bar = ProgressBar(total=len(wikipedia_lists), echo=echo - 1)
		progress_amount = 0
		the_list = []
		for key, value in wikipedia_lists.items():
			progress_bar.show(amount=progress_amount, text=key)
			if isinstance(value, WikipediaListGuide):
				the_list.append(value.get_list())
			else:
				the_list.append(value)
			progress_amount += 1

		progress_bar.show(amount=progress_amount, text='adding lists complete!')


		if categories is None:
			categories = []
		elif not isinstance(categories, (list, tuple)):
			categories = [categories]

		self._pensieve = Pensieve()
		self.pensieve['categories'] = categories
		self.pensieve['sublists'] = the_list

		self.pensieve.store(
			key='records', precursors=['sublists', 'categories'],
			function=lambda x: merge_records(
				lists_of_records=[y.records for y in x['sublists']], additional_categories=x['categories'],
				echo=echo
			),
			evaluate=False
		)

		self.pensieve.store(
			key='data',
			precursors='records',
			function=get_records_data,
			evaluate=False
		)

	_STATE_ATTRIBUTES_ = ['_pensieve']

	def __getstate__(self):
		return {key: getattr(self, key) for key in self._STATE_ATTRIBUTES_}

	def __setstate__(self, state):
		for key, value in state.items():
			setattr(self, key, value)

	def __repr__(self):
		return '\n'.join([repr(x) for x in self.pensieve['sublists']])

	@property
	def pensieve(self):
		"""
		:rtype: Pensieve
		"""
		return self._pensieve

	@property
	def lists(self):
		"""
		:rtype: list[WikipediaList]
		"""
		return self.pensieve['sublists']

	@property
	def records(self):
		"""
		:rtype: list[dict[str]]
		"""
		return self.pensieve['records']

	@property
	def data(self):
		"""
		:rtype: DataFrame
		"""
		return self.pensieve['data']



SECTOR_CLASSIFICATIONS = {
	'10': 'cyclical',  # energy  https://www.fool.com/investing/general/2012/09/20/the-basics-of-the-energy-sector.aspx
	'15': 'cyclical',  # materials https://www.fool.com/investing/general/2012/09/19/an-investors-guide-to-the-materials-sector.aspx
	'20': 'cyclical',  # industrials https://www.fool.com/investing/general/2012/09/19/the-basics-of-the-industrials-sector.aspx
	'25': 'sensitive',  # consumer discretionary https://marketrealist.com/2016/09/difference-consumer-discretionary-consumer-staples/
	'30': 'defensive',  # consumer staples https://www.thebalance.com/what-are-defensive-sectors-2466812
	'35': 'defensive',  # health care https://www.privatebanking.societegenerale.com/en/media/investment-strategy/equity-solutions/equity-nutshell/cyclical-defensive-stocks/
	'40': 'cyclical',  # financials https://fingerlakeswm.com/sector-investing-visualized/
	'45': 'sensitive',  # information technology https://www.investopedia.com/articles/stocks/10/primer-on-the-tech-industry.asp
	'50': 'cyclical',  # communication https://www.fidelity.com/bin-public/060_www_fidelity_com/documents/fidelity/sectors-are-shifting_fidelity.pdf
	'5010': 'defensive',  # telecom  https://www.fidelity.com/bin-public/060_www_fidelity_com/documents/fidelity/sectors-are-shifting_fidelity.pdf
	'55': 'defensive',  # utilities https://fingerlakeswm.com/sector-investing-visualized/
	'60': 'defensive'  # real estate https://www.thebalance.com/what-are-defensive-sectors-2466812
}


KEY_LENGTHS = sorted(list(set([len(key) for key in SECTOR_CLASSIFICATIONS.keys()])), reverse=True)


def classify_sector(gics):
	gics = str(gics)
	for key_length in KEY_LENGTHS:
		if len(gics) >= key_length:
			if gics[:key_length] in SECTOR_CLASSIFICATIONS:
				return SECTOR_CLASSIFICATIONS[gics[:key_length]]

	return 'unknown'



import pycountry
import pandas as pd
import re

from chronometry.progress import ProgressBar
from ravenclaw.wrangling import join_wisely

from linguistics.similarity import get_similarity
from .Country import Country

def _get_word_set(x):
	"""
	:type x: str
	:rtype: set of str
	"""
	try:
		x = x.replace("'s", "")
		result = set(re.split(pattern='[^a-zA-Z]+', string=x.lower()))
	except:
		return set()

	for common_words in ['of', 'and']:
		try:
			result.remove(common_words)
		except:
			pass
	return result


# create a dataframe of countries
class Countries:
	_NAME_COLUMNS = ['common_name', 'name', 'official_name']
	def __init__(self):
		self._countries_df = pd.DataFrame(data=[self._country_to_dict(country) for country in pycountry.countries])
		records = self._countries_df.to_dict(orient='records')
		self._alpha2_dict = {record['alpha_2']:record for record in records}
		self._alpha3_dict = {record['alpha_3']:record for record in records}
		self._name_dict = {record['name']:record for record in records}
		# lower letters:
		"""
		for col in self._NAME_COLUMNS:
			self._countries_df[f'{col}_lower_split'] = self._countries_df[col].apply(_get_word_set)
		"""

	@staticmethod
	def _country_to_dict(country):
		"""
		:type country: pycountry.countries.data_class
		:rtype: dict
		"""
		return country.__dict__['_fields']

	@property
	def data(self):
		return self._countries_df.copy()

	def convert_code_to_dict(self, alpha2=None, alpha3=None):
		if alpha2 is not None:
			return self._alpha2_dict[alpha2.upper()]
		elif alpha3 is not None:
			return self._alpha3_dict[alpha3.upper()]
		else:
			raise ValueError(f'either alpha2 or alphe3 should be given')


	def find_country(self, name, echo=False, _country_data=None):

		df = self.data if _country_data is None else _country_data

		scores = pd.DataFrame()
		for col in df.columns:
			if name in list(df[col].values):
				# no need to score names and stuff, just find the country
				result = dict(df[df[col]==name].iloc[0,:])
				result['match_score'] = 1
				return Country(result)
			else:
				scores[col] = df[col].apply(lambda x:get_similarity(
					s1=x, s2=name, case_sensitivity=0.1, first_char_weight=1.0
				))
		max_scores = scores.max(axis=1)
		df['match_score'] = max_scores
		if echo: print(df.sort_values(by='match_score', ascending=False).head())
		result = dict(df.iloc[max_scores.idxmax()])
		return Country(dictionary=result)

	def find_countries(self, names, echo=False, echo_text=''):

		country_data = self.data
		result = list(ProgressBar.map(
			iterable=names,
			function=lambda x:self.find_country(name=x, echo=False, _country_data=country_data),
			echo=echo - 1, text=echo_text
		))
		return result

	def convert_to_alpha_2(self, data, country_col, echo=False, echo_text=''):
		"""
		converts a country column to alpha_2 codes
		:type data: pd.DataFrame
		:type country_col: str
		:rtype: pd.DataFrame
		"""
		country_data = self.data
		country_data['result'] = country_data['alpha_2'].values
		country_data.columns = [f'__{column}__' for column in country_data.columns]
		left = data.copy()
		left['__order__'] = range(left.shape[0])
		country_data.head()
		result_list = []
		for column in country_data.columns:
			if column not in ['__result__', '__order__']:
				joined_data = join_wisely(
					left=left, right=country_data[[column, '__result__']],
					left_on=country_col, right_on=column, remove_duplicates=False
				)
				if joined_data['both'].shape[0] > 0:
					# remove the column that was used for matching (like __alpha_3__) and put the rest in the result_list
					both = joined_data['both']
					left = joined_data['left_only']
					result_list.append(both.drop(axis=1, labels=column))
					if echo: print(f'joined:{both.shape[0]} using {column.strip("_")} and {left.shape[0]} remains ...')
					# keep anything that is left for future joins


		# put all the matched results into one dataset, matching score is 1
		most_of_the_result = pd.concat(result_list)
		most_of_the_result['alpha_2_match_score'] = 1

		# use find countries on the rest:
		countries_left = self.find_countries(names=list(left[country_col]), echo=echo - 1, echo_text=echo_text)
		left['__result__'] = [country['alpha_2'] for country in countries_left]
		left['alpha_2_match_score'] = [country['match_score'] for country in countries_left]

		result = pd.concat([most_of_the_result, left])
		"""
		:type result: pd.DataFrame
		"""
		result.sort_values(by='__order__', inplace=True)
		result[country_col] = result['__result__'].values
		# avoid duplicates
		result = result[~result['__order__'].duplicated()]
		result.drop(axis=1, labels=['__result__', '__order__'], inplace=True)
		result.index = data.index
		return result




from .SpecialPage import SpecialPage

from silverware import Link
from silverware import Spoon
from ravenclaw.wrangling import standardize_columns


def get_link(link):
	if isinstance(link, Link):
		url = link.url
	elif isinstance(link, list):
		if len(link) > 0:
			link = link[0]
			if isinstance(link, Link):
				url = link.url
			else:
				return None
		else:
			return None
	elif isinstance(link, str):
		url = link
		link = Link(url=link, text='')
	else:
		return None

	if url.startswith('https://en.wikipedia.org/wiki/') and \
			'#' not in url and 'index.php?' not in url and '/wiki/file:' not in url.lower():
		return link
	else:
		return None


def get_list(base_url, sections_of_interest, first_child, exclude_pages):
	"""
	:type base_url: str
	:type sections_of_interest: Spoon
	:type first_child: bool
	:rtype: list[str]
	"""
	the_list = sections_of_interest.filter(name='li').filter(
		name='a', first_only=True, first_child=first_child, ignore_attributes={'class': 'image'}
	).find_links(base_url=base_url)
	result = [get_link(x) for x in the_list]
	return [link for link in result if link is not None and link.url not in exclude_pages]


def get_table_links(tables, columns, exclude_pages):
	result = []

	for table in tables:
		column_found = False
		for column in columns:
			if column in table.columns:
				column_found = True
				for x in table[column]:
					link = get_link(x)
					if link is not None and link not in result and link.url not in exclude_pages:
						result.append(link)
		if not column_found:
			# find similar column
			for column in columns:
				for table_column in table.columns:
					if table_column.startswith(column):
						for x in table[table_column]:
							link = get_link(x)
							if link is not None and link not in result and link.url not in exclude_pages:
								result.append(link)
						break

	return result


def get_page_lists(representation, data_type, tables, columns, base_url, sections_of_interest, exclude_pages):
	list_data = None
	table_data = None
	if representation == 'list':
		list_data = {data_type: get_list(
			base_url=base_url, sections_of_interest=sections_of_interest, first_child=True,
			exclude_pages=exclude_pages
		)}
	elif representation == 'table':
		table_data = {
			data_type: get_table_links(tables=tables, columns=column_names, exclude_pages=exclude_pages)
			for data_type, column_names in columns.items()
		}
	else:
		list_data = {data_type: get_list(
			base_url=base_url, sections_of_interest=sections_of_interest, first_child=True,
			exclude_pages=exclude_pages
		)}
		table_data = {
			data_type: get_table_links(tables=tables, columns=column_names, exclude_pages=exclude_pages)
			for data_type, column_names in columns.items()
		}
	return {'list_data': list_data, 'table_data': table_data}


def combine_list_and_table_data(list_data, table_data):
	list_data = list_data or {}
	table_data = table_data or {}

	result = {}

	for data_type in set(list_data.keys()).union(table_data.keys()):
		if data_type in list_data and data_type in table_data:
			list_links = list_data[data_type]
			table_links = table_data[data_type]
			if len(list_links) > len(table_links):
				result[data_type] = list_links
				for link in table_links:
					if link not in result[data_type]:
						result[data_type].append(link)
			else:
				result[data_type] = table_links
				for link in list_links:
					if link not in result[data_type]:
						result[data_type].append(link)
		elif data_type in list_data:
			list_links = list_data[data_type]
			result[data_type] = list_links
		else:
			table_links = table_data[data_type]
			result[data_type] = table_links
	return result


class ListPage(SpecialPage):
	def __init__(
			self, representation, columns=None, data_type=None, url=None, wikipedia=None, page=None, sections=None,
			exclude_pages=None
	):
		super().__init__(url=url, wikipedia=wikipedia, page=page, sections=sections)
		self.pensieve['exclude_pages'] = exclude_pages or []
		self.pensieve['list_representation'] = representation
		self.pensieve['list_table_columns'] = columns
		self.pensieve['list_data_type'] = data_type
		self.pensieve.store(
			key='tables',
			precursors=['sections_of_interest', 'base_url'],
			evaluate=False,
			function=lambda x: [
				standardize_columns(data=table)
				for table in Spoon.filter(
					soup=x['sections_of_interest'], name='table', attributes={'class': 'wikitable'}
				).read_tables(base_url=x['base_url'], parse_links=True)
			]
		)
		self.pensieve.store(
			key='list_and_table_links',
			precursors=[
				'list_representation', 'list_data_type', 'tables', 'list_table_columns',
				'base_url', 'sections_of_interest', 'exclude_pages'
			],
			function=lambda x: get_page_lists(
				representation=x['list_representation'],
				data_type=x['list_data_type'],
				tables=x['tables'],
				columns=x['list_table_columns'],
				base_url=x['base_url'],
				sections_of_interest=x['sections_of_interest'],
				exclude_pages=x['exclude_pages']
			)
		)
		self.pensieve.store(
			key='page_links',
			precursors='list_and_table_links',
			function=lambda x: combine_list_and_table_data(list_data=x['list_data'], table_data=x['table_data'])
		)

	@property
	def page_links(self):
		"""
		:rtype: dict[str, list[Link]]
		"""
		return self.pensieve['page_links']



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

SPACE_OBJECT = 'space_object'


def get_space_object_guides(wikipedia=None, echo=0):
	return {
		'solar_system_objects': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, categories=[SPACE_OBJECT, 'solar_system_object'],
			data_type=SPACE_OBJECT, echo=echo,
			urls='https://en.wikipedia.org/wiki/List_of_Solar_System_objects',
			representation='list'
		),
		'exoplanets': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, categories=[SPACE_OBJECT, 'exoplanet'],
			data_type=SPACE_OBJECT, echo=echo,
			urls='https://en.wikipedia.org/wiki/List_of_exoplanets_(full)',
			representation='table',
			columns={SPACE_OBJECT: ['name']}
		),
		'stars_by_constellation': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, categories=[SPACE_OBJECT, 'star'],
			data_type=SPACE_OBJECT, echo=echo,
			urls='https://en.wikipedia.org/wiki/Lists_of_stars_by_constellation',
			representation='table',
			columns={SPACE_OBJECT: ['name']}
		),
		'constellations': WikipediaListGuide(
			list_type='list', wikipedia=wikipedia, categories=[SPACE_OBJECT, 'constellation'],
			data_type=SPACE_OBJECT, echo=echo,
			urls='https://en.wikipedia.org/wiki/IAU_designated_constellations',
			representation='table',
			columns={SPACE_OBJECT: ['constellation']}
		),
		'galaxies': WikipediaListGuide(
			list_type='list', wikipedia=wikipedia, categories=[SPACE_OBJECT, 'galaxy'],
			data_type=SPACE_OBJECT, echo=echo,
			urls='https://en.wikipedia.org/wiki/List_of_galaxies',
			representation='table',
			columns={SPACE_OBJECT: ['galaxy']}
		)
	}


def get_space_objects(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	list_guides = get_space_object_guides(wikipedia=wikipedia, echo=progress_bar - 1)
	return WikipediaLists(wikipedia_lists=list_guides, categories=[SPACE_OBJECT], echo=progress_bar - 1)



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

WEAPON = 'weapon'


def get_weapons(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			'weapons': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=WEAPON, categories=[WEAPON],
				urls=[
					'https://en.wikipedia.org/wiki/Lists_of_weapons',
					'https://en.wikipedia.org/wiki/List_of_World_War_II_weapons',
					'https://en.wikipedia.org/wiki/Lists_of_rockets',
					'https://en.wikipedia.org/wiki/Lists_of_swords',
					'https://en.wikipedia.org/wiki/Draft:List_of_service_rifles',
					'https://en.wikipedia.org/wiki/List_of_assault_rifles',
					'https://en.wikipedia.org/wiki/List_of_aircraft_carriers_of_the_United_States_Navy',
					'https://en.wikipedia.org/wiki/List_of_active_United_States_military_aircraft',
					'https://en.wikipedia.org/wiki/List_of_submarines_of_the_United_States_Navy',
					'https://en.wikipedia.org/wiki/List_of_active_Indian_military_aircraft',
					'https://en.wikipedia.org/wiki/Lists_of_armoured_fighting_vehicles'
				],
				exclude_urls=[
					'https://en.wikipedia.org/wiki/List_of_firearms',
					'https://en.wikipedia.org/wiki/List_of_National_Treasures_of_Japan_(crafts:_swords)',
					'https://en.wikipedia.org/wiki/List_of_fictional_swords',
					'https://en.wikipedia.org/wiki/Classification_of_swords'
				],
				columns={WEAPON: ['model', 'name', 'type', 'name_designation', 'firearm', 'aircraft']}
			),
			'tanks': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=WEAPON, categories=[WEAPON],
				urls=[
					'https://en.wikipedia.org/wiki/List_of_main_battle_tanks_by_generation',
					'https://en.wikipedia.org/wiki/List_of_tanks_of_the_Soviet_Union',
					'https://en.wikipedia.org/wiki/List_of_tanks_of_the_United_Kingdom',
					'https://en.wikipedia.org/wiki/List_of_main_battle_tanks_by_country'
				],
				columns={WEAPON: ['model', 'name', 'type', 'name_designation', 'firearm', 'aircraft']}
			)
		}
	)



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar


HISTORIC_EVENT = 'historic_event'

def get_historic_events(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)

	battles_and_disasters = {
		'by_period': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle'],
			representation=None,
			columns={HISTORIC_EVENT: ['name', 'battle']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_before_301',
				'https://en.wikipedia.org/wiki/List_of_battles_301–1300',
				'https://en.wikipedia.org/wiki/List_of_battles_1301%E2%80%931600',
				'https://en.wikipedia.org/wiki/List_of_battles_1601%E2%80%931800',
				'https://en.wikipedia.org/wiki/List_of_battles_1801%E2%80%931900',
				'https://en.wikipedia.org/wiki/List_of_battles_1901%E2%80%932000',
				'https://en.wikipedia.org/wiki/List_of_battles_since_2001',


			]
		),
		'american_revolutionary_war': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle', 'american_revolution'],
			representation='table',
			columns={HISTORIC_EVENT: ['name', 'battle']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_American_Revolutionary_War_battles'
			]
		),
		'american_civil_war_battles': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle', 'american_civil_war'],
			representation='table',
			columns={HISTORIC_EVENT: ['name', 'battle']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_American_Civil_War_battles'
			]
		),
		'napoleonic_battles': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle', 'napoleonic_war'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_Napoleonic_battles'
			]
		),
		'world_war_1': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle', 'world_war_1'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_military_engagements_of_World_War_I'
			]
		),
		'world_war_2': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle', 'world_war_2'],
			representation='list',
			urls=[
				'https://en.wikipedia.org/wiki/List_of_military_engagements_of_World_War_II'
			]
		),
		'battles_by_casualties': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle'],
			representation='table',
			columns={HISTORIC_EVENT: ['battle', 'operation']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_by_casualties'
			]
		),
		'sieges_by_casualties': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'siege', 'battle'],
			representation='table',
			columns={HISTORIC_EVENT: ['siege']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_by_casualties'
			]
		),
		'wars_by_casualties': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'war'],
			representation='table',
			columns={HISTORIC_EVENT: ['conflict', 'name_of_conflict']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_by_casualties',
				'https://en.wikipedia.org/wiki/List_of_wars:_2003%E2%80%93present'
			]
		),
		'disasters': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'disaster'],
			representation='table',
			columns={HISTORIC_EVENT: ['event']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_wars_and_anthropogenic_disasters_by_death_toll'
			]
		),
		'disasters_2': WikipediaListGuide(
			list_type='list_of_lists', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'disaster'],
			representation=None,
			columns={HISTORIC_EVENT: ['event']},
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_disasters'
			],
			sections=['Natural disasters'],
			exclude_list_pages=[
				'https://en.wikipedia.org/wiki/List_of_accidents_and_incidents_involving_commercial_aircraft',
				'https://en.wikipedia.org/wiki/List_of_accidents_and_incidents_involving_military_aircraft',
				'https://en.wikipedia.org/wiki/List_of_airship_accidents',

			]
		),
		'massacres': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'massacre'],
			representation='table',
			columns={HISTORIC_EVENT: ['name']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_events_named_massacres',
				'https://en.wikipedia.org/wiki/List_of_massacres_in_the_United_States'
			]
		),
		'genocides': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'genocide'],
			representation='table',
			columns={HISTORIC_EVENT: ['event']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_genocides_by_death_toll'
			]
		),
		'earthquakes': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'natural_disaster', 'earthquake'],
			representation='table',
			columns={HISTORIC_EVENT: ['event']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_21st-century_earthquakes',
				'https://en.wikipedia.org/wiki/List_of_20th-century_earthquakes'
			]
		),
		'natural_disasters': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'disaster', 'natural_disaster'],
			representation='table',
			columns={HISTORIC_EVENT: ['event']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_natural_disasters_by_death_toll'
			]
		),
		'terrorist_attacks': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'disaster', 'terrorist_attack'],
			representation='table',
			columns={HISTORIC_EVENT: ['name']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_and_other_violent_events_by_death_toll'
			],
			sections=['Terrorist attacks']
		),
		'riots': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'disaster', 'riot'],
			representation='table',
			columns={HISTORIC_EVENT: ['name']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_and_other_violent_events_by_death_toll'
			],
			sections=['Mass unrest, riots and pogroms']
		),
		'rail_accidents': WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'disaster', 'rail_accident'],
			representation='table',
			columns={HISTORIC_EVENT: ['name']},
			urls=[
				'https://en.wikipedia.org/wiki/List_of_battles_and_other_violent_events_by_death_toll'
			],
			sections=['Mass unrest, riots and pogroms']
		),
	}

	by_country_dictionary = {
		'swedish_battle': 'https://en.wikipedia.org/wiki/List_of_Swedish_battles',
		'american_battle': 'https://en.wikipedia.org/wiki/List_of_battles_with_most_United_States_military_fatalities',
		'chinese_battle': 'https://en.wikipedia.org/wiki/List_of_Chinese_wars_and_battles',
		'french_battle': [
			'https://en.wikipedia.org/wiki/List_of_battles_involving_France_in_the_Middle_Ages',
			'https://en.wikipedia.org/wiki/List_of_battles_involving_France_in_the_Renaissance',
			'https://en.wikipedia.org/wiki/List_of_battles_involving_France_in_the_Ancien_R%C3%A9gime',
			'https://en.wikipedia.org/wiki/List_of_battles_involving_France_in_modern_history'
		],
		'indian_battle': 'https://en.wikipedia.org/wiki/List_of_Indian_battles',
		'japanese_battle': 'https://en.wikipedia.org/wiki/List_of_Japanese_battles',
		'korean_battle': 'https://en.wikipedia.org/wiki/List_of_Korean_battles',
		'prussian_battle': 'https://en.wikipedia.org/wiki/Wars_and_battles_involving_Prussia',
		'roman_war': 'https://en.wikipedia.org/wiki/List_of_Roman_wars_and_battles',
		'byzantine_battle': 'https://en.wikipedia.org/wiki/List_of_Byzantine_battles',
		'middle_east_conflict': 'https://en.wikipedia.org/wiki/List_of_modern_conflicts_in_the_Middle_East'
	}

	by_country = {
		country: WikipediaListGuide(
			list_type='list', data_type=HISTORIC_EVENT, echo=progress_bar - 1,
			wikipedia=wikipedia, categories=[HISTORIC_EVENT, 'battle', country],
			representation=None,
			columns={HISTORIC_EVENT: ['battle_or_siege', 'event', 'battle']},
			urls=url
		)
		for country, url in by_country_dictionary.items()
	}

	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={**battles_and_disasters, **by_country}
	)

